local MAJOR_VERSION = "Dock-1.0"
local MINOR_VERSION = 200612111

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

local ON_UPDATE_THROTTLE = .1

local Dock = {
    orientation = "HORIZONTAL",
    padding = 0,
    numFrames = 0,
}

local AceEvent

local function newDock(self,o)
    o = o or CreateFrame("Frame")
    local index = getmetatable(o).__index
    setmetatable(o,{__index = function(t,k)
        return Dock[k] or index[k]
    end})
    local slave = CreateFrame("Frame",nil,o)
    o.slave = slave
    o:EnableAutoResize()
    --o:CreateDebugger()
    return o
end

local function onSlaveSizeChanged(self, w, h)
    local p = self:GetParent()
    p:SetWidth(w)
    p:SetHeight(h)
end

function Dock:EnableAutoResize()
    self.slave:SetScript("OnSizeChanged",onSlaveSizeChanged)
end

function Dock:DisableAutoResize()
    self.slave:SetScript("OnSizeChanged",nil)
end

function Dock:AddFrame(frame,atFront)
    self.frames = self.frames or {}
    if self:GetFramePosition(frame) then return end
    
    if atFront then
        table.insert(self.frames,1,frame)
    else
        table.insert(self.frames,frame)
    end
    self.numFrames = #(self.frames)
    self:UpdatePositions()
    
end

function Dock:RemoveFrame(frame)
    local i = self:GetFramePosition(frame)
    if i then
        table.remove(self.frames,i)
        self.numFrames = #(self.frames)
        if self.numFrames == 0 then
            self.slave:ClearAllPoints()
            self.slave:SetWidth(1)
            self.slave:SetHeight(1)
        else
            self:UpdatePositions()
        end
    end

end

function Dock:GetFramePosition(f)
    for i,frame in ipairs(self.frames) do
        if frame == f then
            return i
        end
    end
end

function Dock:UpdatePositions()
    
    if self.numFrames == 0 then
        
        self:DisableAutoResize()
        
        if self.orientation == "HORIZONTAL" then
            self:SetWidth(1)
            self.slave:SetPoint("LEFT",self,0,0)
            self.slave:SetPoint("RIGHT",self,0,0)
        elseif self.orientation == "VERTICAL" then
            self:SetHeight(1)
            self.slave:SetPoint("TOP",self,0,0)
            self.slave:SetPoint("BOTTOM",self,0,0)
        end
        
    else
        
        self:EnableAutoResize()
        
        local paddingX,paddingY,point,relPoint = 0,0
        if self.orientation == "HORIZONTAL" then
            point,relPoint,paddingX = "LEFT","RIGHT",self.padding
        elseif self.orientation == "VERTICAL" then
            point,relPoint,paddingY = "TOP","BOTTOM",-self.padding
        end        
        
        local prev
        for i,frame in ipairs(self.frames) do
            if i == 1 then
                frame:SetPoint(point,self,point,0,0)
            else
                frame:SetPoint(point,prev,relPoint,paddingX,paddingY)
            end
            prev = frame
        end
        
        local slave = self.slave
        if self.orientation == "HORIZONTAL" then
            slave:SetPoint("LEFT",self,0,0)
            slave:SetPoint("RIGHT",prev,0,0)
            slave:SetHeight(self.height or 1)
        elseif self.orientation == "VERTICAL" then
            slave:SetPoint("TOP",self,0,0)
            slave:SetPoint("BOTTOM",prev,0,0)
            slave:SetWidth(self.width or 1)
        end
    end
    
    if AceEvent then AceEvent:TriggerEvent("DOCK_UPDATE_FRAME_ORDER",self,self.frames) end    
end

function Dock:StartUpdates()
    self:SetScript("OnUpdate",self.OnUpdate)
end

function Dock:StopUpdates()
    self:SetScript("OnUpdate",nil)
end

function Dock:OnUpdate(elapsed)
    
    local dragFrame = self.dragFrame
    if not dragFrame then self:StopUpdates() return end
    local x,y = GetCursorPosition()
    local axis,moveLeft,moveRight
    local frame = self.frames[dragFrame]
    local s = frame:GetEffectiveScale()
    x,y = x/s,y/s
    local cx,cy = frame:GetCenter()
    local w,h = 0.5 * frame:GetWidth(), 0.5 * frame:GetHeight()
    local padding = self.padding
    
    if self.orientation == "HORIZONTAL" then
        axis = x
        moveLeft = dragFrame == 1 and -1 or cx - w - padding
        moveRight = dragFrame == self.numFrames and -1 or cx + w + padding
    elseif self.orientation == "VERTICAL" then
        axis = -y
        moveLeft = -(dragFrame == 1 and 1 or cy + h + padding)
        moveRight = -(dragFrame == self.numFrames and 1 or cy - h - padding)
    end
    
    if self.debugFrame then self.debugFrame.text:SetText(string.format("axis: %.2f\nmoveLeft: %.2f\nmoveRight: %.2f\nwidth: %.2f",axis,moveLeft,moveRight,self:GetWidth())) end
    
    if axis <= moveLeft and moveLeft ~= -1 then
        self.frames[dragFrame],self.frames[dragFrame-1] = self.frames[dragFrame-1],self.frames[dragFrame]
        self.dragFrame = dragFrame - 1
        self:UpdatePositions()
    elseif axis >= moveRight and moveRight ~= -1 then
        self.frames[dragFrame],self.frames[dragFrame+1] = self.frames[dragFrame+1],self.frames[dragFrame]
        self.dragFrame = dragFrame + 1
        self:UpdatePositions()
    end
end

function Dock:StartDragging(frame)
    if not self.dragFrame then
        local i = self:GetFramePosition(frame)
        if i then
            self.dragFrame = i
            self:StartUpdates()
        end
    end
end

function Dock:StopDragging()
    self.dragFrame = nil
    self:StopUpdates()
end

--[[function Dock:CreateDebugger()
    if self.debugFrame then return end
    self.debugFrame = Tard:CreateDebugFrame(120,65)
end
]]

local function external(self, major, instance)
    if major == "AceEvent-2.0" then
        AceEvent = instance
    end
end

local DockLib = { new = newDock }
AceLibrary:Register(DockLib, MAJOR_VERSION, MINOR_VERSION,nil,nil,external)
DockLib = AceLibrary(MAJOR_VERSION)